<?php
/**
 * Created by PhpStorm.
 * User: crose
 * Date: 3/21/16
 * Time: 9:29 AM
 */

namespace qfq;
//use qfq;

require_once(__DIR__ . '/../../qfq/BodytextParser.php');
require_once(__DIR__ . '/../../qfq/exceptions/UserException.php');


class BodytextParserTest extends \PHPUnit_Framework_TestCase {
    public function testProcessPlain() {
        $btp = new BodytextParser();

        // Simple row, nothing to remove
        $given = "10.sql = SELECT 'Hello World'";
        $expected = $given;
        $result = $btp->process($given);
        $this->assertEquals($expected, $result);

        // Several rows, remove all but one
        $given = "\n#some comments\n10.sql = SELECT 'Hello World'\n\n    \n   #more comment";
        $expected = "10.sql = SELECT 'Hello World'";
        $result = $btp->process($given);
        $this->assertEquals($expected, $result);

        // Several rows, all to remove
        $given = "\n#some comments\n\n\n    \n   #more comment";
        $expected = "";
        $result = $btp->process($given);
        $this->assertEquals($expected, $result);

        // Join a line
        $given = "\n10.sql = SELECT 'Hello World',\n'more content'\n      WHERE help=1";
        $expected = "10.sql = SELECT 'Hello World', 'more content' WHERE help=1";
        $result = $btp->process($given);
        $this->assertEquals($expected, $result);

        // Join several lines, incl. form
        $given = "\n10.sql = SELECT 'Hello World',\n'more content'\n      WHERE help=1\n 20.head = <table>\n 30.sql     =   SELECT\n col1,\n col2, \n col3\n  # Query stops here\nform = Person\n";
        $expected = "10.sql = SELECT 'Hello World', 'more content' WHERE help=1\n20.head = <table>\n30.sql     =   SELECT col1, col2, col3\nform = Person";
        $result = $btp->process($given);
        $this->assertEquals($expected, $result);

        // Nested expression: one
        $given = "10{\nsql = SELECT 'Hello World'}";
        $expected = "10.sql = SELECT 'Hello World'";
        $result = $btp->process($given);
        $this->assertEquals($expected, $result);

        // Nested expression, one, added some white spaces
        $given = "\n\n10 { \n \n   sql    =    SELECT 'Hello World'     \n\n    }\n\n";
        $expected = "10.sql    =    SELECT 'Hello World'";
        $result = $btp->process($given);
        $this->assertEquals($expected, $result);

        // Nested expression: multiple, simple
        $given = "10.sql = SELECT 'Hello World'\n20 {\nsql='Hello world2'\n}\n30 {\nsql='Hello world3'\n}\n";
        $expected = "10.sql = SELECT 'Hello World'\n20.sql='Hello world2'\n30.sql='Hello world3'";
        $result = $btp->process($given);
        $this->assertEquals($expected, $result);

        // Nested expression: complex
        $given = "10.sql = SELECT 'Hello World'\n20 {\nsql='Hello world2'\n30 {\n sql=SELECT 'Hello World3'\n40 { sql = SELECT 'Hello World4'\n}  \n}\n}";
        $expected = "10.sql = SELECT 'Hello World'\n20.sql='Hello world2'\n20.30.sql=SELECT 'Hello World3'\n20.30.40.sql = SELECT 'Hello World4'";
        $result = $btp->process($given);
        $this->assertEquals($expected, $result);

        // form=...., {{ }}
        $given = "10.sql = SELECT 'Hello World'\nform = {{form:S}}\n20.sql = SELECT 'Hello World2'\n30 { \nsql=SELECT 'Hello World'\n}\n   form=Person\n";
        $expected = "10.sql = SELECT 'Hello World'\nform = {{form:S}}\n20.sql = SELECT 'Hello World2'\n30.sql=SELECT 'Hello World'\nform=Person";
        $result = $btp->process($given);
        $this->assertEquals($expected, $result);

        // Nested: open bracket alone
        $given = "10.sql = SELECT 'Hello World'\n20\n{\nhead=test\n}\n30.sql = SELECT 'Hello World'\n";
        $expected = "10.sql = SELECT 'Hello World'\n20.head=test\n30.sql = SELECT 'Hello World'";
        $result = $btp->process($given);
        $this->assertEquals($expected, $result);

        // Nested: unclosed open bracket
        $given = "10.sql = SELECT 'Hello World'\n20 {\n30.sql = SELECT 'Hello World'\n";
        $expected = "10.sql = SELECT 'Hello World'\n20 {\n30.sql = SELECT 'Hello World'";
        $result = $btp->process($given);
        $this->assertEquals($expected, $result);

    }

    /**
     * @expectedException \qfq\UserException
     *
     */
    public function testProcessException() {
        $btp = new BodytextParser();

        // Nested: unclosed open bracket
        $btp->process("10.sql = SELECT 'Hello World'\n } \n30.sql = SELECT 'Hello World'\n");

    }

}
